#ifndef SRC_CORE_FILE_CONTROLLER_HPP_
#define SRC_CORE_FILE_CONTROLLER_HPP_

#include <unordered_map>
#include <vector>
#include <set>
#include <map>
#include "src/core/controller.hpp"
#include "src/core/settings.hpp"
#include "src/core/rom.hpp"
#include "src/ui/ui-state.hpp"

namespace FileController {

	extern const AppSettings &loadAppSettings();
	extern void saveAppSettings( const AppSettings &settings );

	extern const std::vector<ROM> &loadRomList();
	extern void saveRomList( const std::vector<ROM> &roms );

	extern const std::vector<RomSource> &loadRomSources();
	extern void saveRomSources( const std::vector<RomSource> &sources );

	extern const std::vector<fs::path> &loadSavedRoms();
	extern void saveSavedRoms( const std::vector<fs::path> &savedRoms );

	extern const std::set<string> &loadTags();
	extern void saveTags( const std::set<string> &tags );

	extern const std::map<string, ControllerProfile> &loadControllerProfiles();
	extern void saveControllerProfiles( const std::map<string, ControllerProfile> &profiles );

	extern const HashMap<Uuid,string> &loadControllerMappings();
	extern void saveControllerMappings( const HashMap<Uuid,string> &mappings );

	extern const UiState &loadUiState();
	extern void saveUiState( const UiState &state );

	extern const std::vector<int> &loadHotkeys();
	extern void saveHotkeys( const std::vector<int> &hotkeys );

	extern const std::map<Uuid,InputMode> &loadInputModes();
	extern void saveInputModes( const std::map<Uuid,InputMode> &modes );

#ifdef _WIN32
	extern void fixFileEncodings();
#endif

}



#endif /* SRC_CORE_FILE_CONTROLLER_HPP_ */
